# -*- coding: utf-8 -*-

import martian
from dolmen.content import Content
from dolmen.app.content import icon
from grokcore.formlib import formlib
from zope.interface import directlyProvides
from zope.browserresource.metaconfigure import icon as register_icon


class ContentIconGrokker(martian.ClassGrokker):
    martian.component(Content)
    martian.directive(icon)

    def execute(self, class_, config, icon, **kw):
        if icon:
            specialized = formlib.most_specialized_interfaces(class_)
            register_icon(config, 'icon', specialized[0], file=icon)
            directlyProvides(class_, specialized[0])
        return True



# -*- coding: utf-8 -*-

import grok

from PIL import Image
from cStringIO import StringIO
from dolmen.file import IImageField
from dolmen.blob import BlobValue
from dolmen.content import IContent
from dolmen.forms.base import IFieldUpdate
from dolmen.thumbnailer import Miniaturizer, IImageMiniaturizer, IThumbnailer


class BlobMiniaturizer(Miniaturizer):
    """Miniaturizer handler for `dolmen.content.IBaseContent` objects.
    It adds a 64*64 scale (square) and stores the thumbnails in blobs.
    """
    grok.context(IContent)

    # We store in blob
    factory = BlobValue

    # We add a new size
    scales = {'large': (700, 700),
              'preview': (400, 400),
              'mini': (250, 250),
              'thumb': (150, 150),
              'small': (128, 128),
              'square': (64, 64),
              }


class SquareThumbnailer(grok.Adapter):
    """A named thumbnailer that will crop images to the given size.
    """
    grok.name('square')
    grok.context(IContent)
    grok.implements(IThumbnailer)

    def scale(self, original, size):
        image = original.copy()

        width, height = image.size

        if width > height:
            delta = width - height
            left = int(delta / 2)
            upper = 0
            right = height + left
            lower = height
        else:
            delta = height - width
            left = 0
            upper = int(delta / 2)
            right = width
            lower = width + upper

        image = image.crop((left, upper, right, lower))
        image.thumbnail(size, Image.ANTIALIAS)
        thumbnailIO = StringIO()
        image.save(thumbnailIO, original.format, quality=90)
        thumbnailIO.seek(0)
        return thumbnailIO


@grok.implementer(IFieldUpdate)
@grok.adapter(IContent, IImageField)
def ThumbnailsGeneration(object, field):
    """Event handler triggering the thumbnail generation
    """
    name = field.__name__
    original = getattr(object, name, None)
    handler = IImageMiniaturizer(object)

    # The image has been deleted if 'original' is None
    if original is None:
        # We delete the thumbnails.
        handler.delete(fieldname=name)
    else:
        # We generate the thumbnails.
        handler.generate(fieldname=name)



# -*- coding: utf-8 -*-

import os.path
import martian

from sys import modules
from dolmen import content
from martian.directive import StoreOnce


def get_absolute_path(filename, pyfile=__file__):
    if os.path.isfile(filename):
        return filename
    path = os.path.join(os.path.dirname(pyfile), filename)
    if not os.path.isfile(path):
        return None
    return path


class FileValueStoreOnce(StoreOnce):
    """Stores the abs. path of the file given as a value.
    """
    def set(self, locals_, directive, value):
        pyfile = modules[locals_['__module__']].__file__
        path = get_absolute_path(value, pyfile)
        if path is None:
            raise martian.error.GrokImportError(
                "Directive %r cannot resolve the file %r." %
                (directive.name, value))
        StoreOnce.set(self, locals_, directive, path)


FILE_PATH_ONCE = FileValueStoreOnce()


class icon(martian.Directive):
    scope = martian.CLASS
    store = FILE_PATH_ONCE
    validate = martian.validateText


icon.set(content.Content, get_absolute_path('content.png'))
icon.set(content.Container, get_absolute_path('container.png'))
icon.set(content.OrderedContainer, get_absolute_path('container.png'))



from dolmen.app.content.directives import icon
from dolmen.app.content.interfaces import IDescriptiveSchema
from dolmen.app.content.thumbnails import BlobMiniaturizer, SquareThumbnailer
from dolmen.app.content.indexes import *



# -*- coding: utf-8 -*-

import unittest, doctest
import dolmen.app.content
import zope.component

from zope.component.testlayer import ZCMLFileLayer
from zope.site.folder import rootFolder
from zope.site.site import LocalSiteManager


class DolmenAppContentLayer(ZCMLFileLayer):

    def setUp(self):
        ZCMLFileLayer.setUp(self)
        zope.component.hooks.setHooks()

        # Set up site
        site = rootFolder()
        site.setSiteManager(LocalSiteManager(site))
        zope.component.hooks.setSite(site)

    def tearDown(self):
        zope.component.hooks.resetHooks()
        zope.component.hooks.setSite()
        ZCMLFileLayer.tearDown(self)


def test_suite():
    suite = unittest.TestSuite()
    readme = doctest.DocFileSuite(
        'README.txt',
        globs={"__name__": "dolmen.app.content"},
        optionflags=(doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS))
    readme.layer = DolmenAppContentLayer(dolmen.app.content)
    suite.addTest(readme)
    return suite



# -*- coding: utf-8 -*-

from zope.i18nmessage import MessageFactory
from zope.schema import TextLine, Text
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.dublincore.interfaces import IDCDescriptiveProperties

_ = MessageFactory('zope')


class IDescriptiveSchema(IAttributeAnnotatable, IDCDescriptiveProperties):
    """Marker interface for a Dolmen content.
    """
    title = TextLine(
        title=_(u"Title"),
        required=True)

    description = Text(
        title=_(u'Description'),
        required=False,
        default=u"")



# -*- coding: utf-8 -*-

import grok
from grok import index
from dolmen.content import IContent
from dolmen.app.site import IDolmen
from dolmen.app.content import IDescriptiveSchema
from zope.index.text.interfaces import ISearchableText


class ContentIndexes(grok.Indexes):
    """Indexes the content type of a ``dolmen.content`` `IContent`.
    """
    grok.site(IDolmen)
    grok.context(IContent)
    content_type = index.Field(attribute='__content_type__')


class DescriptiveIndexes(grok.Indexes):
    """Indexes the attributes of an `IDescriptiveSchema` content.
    """
    grok.site(IDolmen)
    grok.context(IDescriptiveSchema)

    title = index.Text()
    description = index.Text()


class SearchableIndex(grok.Indexes):
    """Indexes the searchable text of an `ISearchableText` content.
    """
    grok.site(IDolmen)
    grok.context(ISearchableText)

    searchabletext = index.Text(attribute="getSearchableText")


class SearchableDescription(grok.Adapter):
    """Provides the needed `ISearchableText` methods for
    `IDescriptiveSchema`.
    """
    grok.implements(ISearchableText)
    grok.context(IDescriptiveSchema)

    def getSearchableText(self):
        return (self.context.title, self.context.description)


__all__ = ['ContentIndexes', 'DescriptiveIndexes',
           'SearchableIndex', 'SearchableDescription']



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.app.content'
version = '1.0b1'
readme = open(join('src', 'dolmen', 'app', 'content', 'README.txt')).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'PIL',
    'dolmen.app.site',
    'dolmen.blob >= 0.5',
    'dolmen.content >= 0.7.0',
    'dolmen.file >= 0.5',
    'dolmen.forms.base',
    'dolmen.thumbnailer >= 0.2.1',
    'grok',
    'grokcore.component',
    'grokcore.formlib',
    'martian',
    'setuptools',
    'zope.annotation',
    'zope.browserresource',
    'zope.dublincore',
    'zope.i18nmessage',
    'zope.index',
    'zope.interface',
    'zope.schema',
    ]

tests_require = [
    'zope.annotation',
    'zope.browserpage',
    'zope.component',
    'zope.i18n',
    'zope.publisher',
    'zope.schema',
    'zope.security',
    'zope.site',
    'zope.traversing',
    ]

setup(name = name,
      version = version,
      description = 'Dolmen applications content utilities',
      long_description = readme[readme.find('\n\n'):] + '\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = 'http://gitweb.dolmen-project.org/',
      download_url = '',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.app'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite="dolmen.app.content",
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)
